/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.shapetype;

import com.vividsolutions.jts.geom.Geometry;
import java.awt.datatransfer.Transferable;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import javax.swing.Action;
import nl.cloudfarming.client.geometry.model.Shape;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import org.geotools.data.FeatureSource;
import org.geotools.data.shapefile.ShapefileDataStore;
import org.geotools.feature.FeatureCollection;
import org.geotools.feature.FeatureIterator;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.type.AttributeDescriptor;
import org.opengis.feature.type.AttributeType;
import org.opengis.feature.type.Name;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

@NbBundle.Messages("ShapeDataObject.shape_name=Unknown")
public class ShapeDataObject extends MultiDataObject {

    public static final String MIMETYPE = "application/x-esri-shape";
    private final InstanceContent ic = new InstanceContent();
    private final Lookup lkp = new AbstractLookup(ic);
    private List<Shape> shapes = null;
    private List<String>  stringAttributes;
    private List<String> integerAttributes;

    public ShapeDataObject(FileObject pf, MultiFileLoader loader) throws DataObjectExistsException, IOException {
        super(pf, loader);
        registerEditor(MIMETYPE, true);
        ic.add(new ShapeImportSource() {

            @Override
            public Set<FileObject> getFiles() {
                return files();
            }
        });
    }

    /**
     * Method to set the secondary files
     *
     * @param secondary the secondary files
     */
    public void setSecondaryFiles(List<FileObject> secondary) {
        for (FileObject fileObject : secondary) {
            registerEntry(fileObject);
        }
    }

    @Override
    protected Node createNodeDelegate() {
        Node delegate = super.createNodeDelegate();
        ic.add(new ShapeCollection(getShapes()));
        return new ShapeDataObjectNode(delegate, getPrimaryFile().getName());
    }

    @Override
    public Lookup getLookup() {
        return new ProxyLookup(super.getLookup(), lkp);
    }

    /**
     * Retrieve all shapes in this shapeDataObject
     *
     * @return
     */
    public final List<Shape> getShapes() {
        if (shapes == null) {
            shapes = new ArrayList<>();
            try {
                ShapefileDataStore shpDataStore = new ShapefileDataStore(getPrimaryFile().getURL());
                String typeName = shpDataStore.getTypeNames()[0];
                FeatureSource featureSource = shpDataStore.getFeatureSource(typeName);

                FeatureCollection result = featureSource.getFeatures();
                FeatureIterator iterator = result.features();
                while (iterator.hasNext()) {
                    SimpleFeature feature = (SimpleFeature) iterator.next();
                    Shape shape = getShape(feature);
                    //
                    // parse all available string attributes
                    //
                    stringAttributes = readAttributes(feature, String.class);
                    //
                    // add all string attributes and their values to the map in the shape object
                    //
                    for (String att : stringAttributes) {
                        int index = feature.getFeatureType().indexOf(att);
                        if (index >= 0) {
                            String value = (String) feature.getAttribute(index);
                            shape.addStringAttribute(att, value);
                        }
                    }
                    //
                    // parse all available Integer attributes
                    //
                    integerAttributes = readAttributes(feature, Integer.class);
                    //
                    // add all string attributes and their values to the map in the shape object
                    //
                    for (String att : integerAttributes) {
                        int index = feature.getFeatureType().indexOf(att);
                        if (index >= 0) {
                            int value = (Integer)feature.getAttribute(index);
                            shape.addIntegerAttribute(att, new Integer(value).doubleValue());
                        }
                    }
                    
                    List<String> doubleValues = readAttributes(feature, Double.class);
                    for(String att : doubleValues){
                        int index = feature.getFeatureType().indexOf(att);
                        if (index >= 0) {
                            Double value = (Double)feature.getAttribute(index);
                            shape.addIntegerAttribute(att, value);
                        }
                    }
                    
                    integerAttributes.addAll(doubleValues);
                    
                    
                    shapes.add(shape);
                }
                iterator.close();
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }
        }
        return shapes;
    }

    /**
     * Reads attributes of type String from specified feature into the
     * attributes List
     *
     * @param feature
     */
    private List<String> readAttributes(SimpleFeature feature, Class clazz) {
        List<String> returnList = new ArrayList<>();
        List<AttributeDescriptor> attributeDescriptors = feature.getFeatureType().getAttributeDescriptors();
        for (AttributeDescriptor attributeDescriptor : attributeDescriptors) {
            // Determine type
            AttributeType attributeType = attributeDescriptor.getType();

            boolean attributeIsString = attributeType.getBinding().equals(clazz);

            if (attributeIsString) {
                // Determine Attributename
                Name name = attributeDescriptor.getName();
                if (name != null) {
                    returnList.add(name.toString());
                }
            }
        }
        return returnList;
    }

    
    /**
     * Get the attributes of type 'Integer' from this shp-file
     *
     * @return The attributes inside the shp-file
     */
    public List<String> getIntegerAtributes() {
        if(integerAttributes == null){
            getShapes();
        }
        return integerAttributes;
    }

    /**
     * Get the attributes of type 'String' from this shp-file
     *
     * @return The attributes inside the shp-file
     */
    public List<String> getStringAttributes() {
        // Read shapes if not already done
        if(stringAttributes == null){
            getShapes();
        }
        return stringAttributes;
    }

    /**
     *
     * @param payload
     * @return
     */
    private Shape getShape(SimpleFeature payload) {

        int attIndexPlotPoly = payload.getFeatureType().indexOf("the_geom");

        Geometry geometry = null;
        if (attIndexPlotPoly >= 0) {
            geometry = (Geometry) payload.getAttribute(attIndexPlotPoly);
        }
        String shapeName = Bundle.ShapeDataObject_shape_name();

        Shape shape = new Shape(geometry);
        shape.setName(shapeName);
        return shape;
    }

    @Override
    protected int associateLookup() {
        return 1;
    }

    /**
     * Root node for the shapefile
     */
    private final class ShapeDataObjectNode extends AbstractNode {

        final Node origional;

        public ShapeDataObjectNode(Node original, String name) {
            super(Children.create(new ShapeChildFactory(), true), new ProxyLookup(original.getLookup(), Lookups.singleton(new SimpleLayer(name))));
            this.origional = original;
            setIconBaseWithExtension("nl/cloudfarming/client/shapetype/file_extension_shp.png");
            setDisplayName(name);
        }

        @Override
        public Action[] getActions(boolean context) {
            return origional.getActions(context);
        }

        @Override
        public PasteType getDropType(Transferable t, int action, int index) {
            return null;
        }
    }

    /**
     * Creates shape nodes for the simple features within the specified
     * shapefile
     */
    private final class ShapeChildFactory extends ChildFactory<Shape> {

        @Override
        protected boolean createKeys(List<Shape> toPopulate) {
            // Add all features in this Dataobject to the nodelist
            toPopulate.addAll(ShapeDataObject.this.getShapes());
            return true;
        }

        @Override
        protected Node createNodeForKey(Shape key) {
            Node shapeNode = new ShapeNode(key);
            return shapeNode;
        }
    }

    /**
     * Node representing a SimpleFeature (shape) in a shape file
     */
    private final class ShapeNode extends AbstractNode {

        public ShapeNode(Shape shape) {
            super(Children.LEAF, Lookups.fixed(shape, new SimpleLayer(shape.getName())));
            setName(shape.getName());
            setIconBaseWithExtension("nl/cloudfarming/client/shapetype/file_extension_shp.png");
        }

        @Override
        public PasteType getDropType(Transferable t, int action, int index) {
            return null;
        }
    }
}
