/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.shapetype.preview;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.geometry.PreviewPanel;
import nl.cloudfarming.client.geometry.model.GeometryContainer;
import nl.cloudfarming.client.geometry.model.Shape;
import nl.cloudfarming.client.geometry.wizard.ImportShapeFilePicker;
import nl.cloudfarming.client.shapetype.ShapeDataObject;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.netbeans.api.visual.action.ActionFactory;
import org.netbeans.api.visual.model.*;
import org.openide.WizardDescriptor;
import org.openide.WizardValidationException;
import org.openide.util.HelpCtx;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;

/**
 * Default PreviewProvider
 *
 * @author Gerben Feenstra
 */
public class ShapePreviewController extends PreviewPanel {
    
    private ASBodyPanel shapePreviewASBodyPanel;
    private WizardDescriptor descriptor;
    private ShapeDataObject dataObject;
    private final ObjectScene scene = new ObjectScene();
    private final SceneListener sceneListener = new SceneListener();
    private MapLegendPanel mapLegendPanel;
    private ShapePropertiesPanel shapePropertiesPanel = new ShapePropertiesPanel();
    private ShapePropertyMappingPanel shapePropertyMappingPanel;
    private PropertyMappingListener propertyMappingListener = new PropertyMappingListener();
    private PreviewPanelPalette previewPanelPalette;
    private ImageIcon titleIcon = new ImageIcon(ImageUtilities.loadImage("nl/cloudfarming/client/shapetype/map_add32.png"));

    @NbBundle.Messages({"shape preview attributes no attribute=None", "shape preview panel title=Shape import preview"})
    public ShapePreviewController(ShapeDataObject dataObject) {
        this.dataObject = dataObject;
        this.previewPanelPalette = new PreviewPanelPalette(Color.red);
    }

    @Override
    public Component getComponent() {
        if (shapePreviewASBodyPanel == null) {
            mapLegendPanel = new MapLegendPanel(previewPanelPalette);

            scene.getActions().addAction(ActionFactory.createZoomAction());
            scene.getActions().addAction(ActionFactory.createPanAction());

            ShapePreviewPanel.Builder builder = new ShapePreviewPanel.Builder();

            // Shape Preview panel
            ShapesPanel leftPanel = new ShapesPanel(dataObject, scene, previewPanelPalette);
            builder.setLeftPanel(leftPanel);

            // Legend panel

            builder.setMapLegendPanel(mapLegendPanel);

            // Shape selectorPanel
            List<String> valueAttributes = dataObject.getIntegerAtributes();
            if (valueAttributes != null && !valueAttributes.isEmpty() && !valueAttributes.contains("")) {
                valueAttributes.add("");
            }
            
            
            List<String> nameAttributes = dataObject.getStringAttributes();
            if (nameAttributes != null && !nameAttributes.isEmpty() && !nameAttributes.contains("")) {
                nameAttributes.add("");
            }

            if(valueAttributes != null && !valueAttributes.isEmpty() && 
                    nameAttributes != null && !nameAttributes.contains(valueAttributes.get(0))) {
                nameAttributes.addAll(valueAttributes);
            }
            shapePropertyMappingPanel = new ShapePropertyMappingPanel();
            shapePropertyMappingPanel.setAvailableNameProperties(nameAttributes);
            shapePropertyMappingPanel.setAvailableValueProperties(valueAttributes);



            shapePropertyMappingPanel.addItemListener(propertyMappingListener);
            builder.setShapePropertyMappingPanel(shapePropertyMappingPanel);

            // Shape properties panel

            builder.setShapePropertiesPanel(shapePropertiesPanel);
            shapePropertiesPanel.addImportTypeListener(propertyMappingListener);


            builder.setMarkAsPanel(new MarkAsPanel());
            ShapePreviewPanel shapePreviewPanel = builder.build();
            shapePreviewPanel.setName(Bundle.shape_preview_panel_title());


            shapePreviewASBodyPanel = new ASBodyPanel(shapePreviewPanel);

            shapePreviewASBodyPanel.setTitle(Bundle.shape_preview_panel_title());
            shapePreviewASBodyPanel.setIcon(titleIcon);
//            /shapePreviewASBodyPanel.setName(Bundle.shape_preview_panel_title());

            scene.addObjectSceneListener(sceneListener, ObjectSceneEventType.OBJECT_SELECTION_CHANGED);
        }
        return shapePreviewASBodyPanel;
    }

    @Override
    public HelpCtx getHelp() {
        return null;
    }

    
    /**
     * Read the setting needed for this panel
     * Reads the property ImportShapeFilePicker.PROP_SELECTED_GEOMETRYCONTAINER
     * This should contain the selected container
     * Checks if this container supports one or multiple Shape.ShapeType's 
     * if it supports one it will set the type of the shapes in the dataObject to the supported type
     * Otherwise will set the type to unknown
     * @param settings The wizardDescriptor
     */
    @Override
    public void readSettings(Object settings) {
        descriptor = (WizardDescriptor) settings;
        GeometryContainer container = (GeometryContainer) descriptor.getProperty(ImportShapeFilePicker.PROP_SELECTED_GEOMETRYCONTAINER);
        List<Shape.ShapeType> shapeTypes = new ArrayList<>();
        for (Shape.ShapeType shapeType : Shape.ShapeType.values()) {
            if (container.supports(shapeType)) {
                shapeTypes.add(shapeType);
            }
        }
        shapePropertiesPanel.setShapeTypes(shapeTypes);

        //Set the Shape.ShapeType of shape
        for (Shape shape : dataObject.getShapes()) {
            if (shapeTypes.size() > 1) {
                shape.setType(Shape.ShapeType.UNKNOWN);
            } else {
                Shape.ShapeType shapeType = shapeTypes.get(0) != null ? shapeTypes.get(0) : Shape.ShapeType.UNKNOWN;
                shape.setType(shapeType);
            }
        }

    }

    @Override
    @NbBundle.Messages({"wizard.no.shapes.error.message=No shapes found in file."})
    public boolean isValid() {
        if (dataObject.getShapes().size() > 0) {
            descriptor.setValid(true);
            descriptor.putProperty(WizardDescriptor.PROP_ERROR_MESSAGE, "");
            return true;
        }
        else {
            descriptor.setValid(false);
            descriptor.putProperty(WizardDescriptor.PROP_ERROR_MESSAGE, Bundle.wizard_no_shapes_error_message());
            return false;
        }
    }

    @Override
    public void validate() throws WizardValidationException {
        if (!isValid()) {
            throw new WizardValidationException(shapePreviewASBodyPanel, WizardDescriptor.PROP_ERROR_MESSAGE, Bundle.wizard_no_shapes_error_message());
        }
    }

    @Override
    public void addChangeListener(ChangeListener l) {
    }

    @Override
    public void removeChangeListener(ChangeListener l) {
    }

    /**
     * Gets the shapes of type Shape.ShapeType.IMPORT
     * @return a list of shapes
     */
    @Override
    public List<Shape> getImportShapes() {
        List<Shape> shapes = new ArrayList<>();
        for (Shape shape : dataObject.getShapes()) {
            if (Shape.ImportType.IMPORT.equals(shape.getImportType())) {
                shapes.add(shape);
            }
        }
        return shapes;
    }

    private class SceneListener implements ObjectSceneListener {

        @Override
        public void objectAdded(ObjectSceneEvent event, Object addedObject) {
        }

        @Override
        public void objectRemoved(ObjectSceneEvent event, Object removedObject) {
        }

        @Override
        public void objectStateChanged(ObjectSceneEvent event, Object changedObject, ObjectState previousState, ObjectState newState) {
        }

        @Override
        public void selectionChanged(ObjectSceneEvent event, Set<Object> previousSelection, Set<Object> newSelection) {
            Shape[] shapes = new Shape[newSelection.size()];
            newSelection.toArray(shapes);
            shapePropertiesPanel.setActiveShape(shapes);
        }

        @Override
        public void highlightingChanged(ObjectSceneEvent event, Set<Object> previousHighlighting, Set<Object> newHighlighting) {
        }

        @Override
        public void hoverChanged(ObjectSceneEvent event, Object previousHoveredObject, Object newHoveredObject) {
        }

        @Override
        public void focusChanged(ObjectSceneEvent event, Object previousFocusedObject, Object newFocusedObject) {
        }
    }

    private class PropertyMappingListener implements ItemListener {

        @Override
        public void itemStateChanged(ItemEvent evt) {
            JComboBox cb = (JComboBox) evt.getSource();
            String boxName = cb.getName();

            // Get the affected item
            Object item = evt.getItem();

            if (evt.getStateChange() == ItemEvent.SELECTED) {
                // Item was just selected
                if (ShapePropertyMappingPanel.PROP_COMBOBOX_MAPPING_NAME.equals(boxName)) {
                    for (Shape shape : dataObject.getShapes()) {
                        shape.setNameFromAttribute((String) item);
                    }
                    Shape[] shapes = new Shape[scene.getSelectedObjects().size()];
                    scene.getSelectedObjects().toArray(shapes);
                    shapePropertiesPanel.setActiveShape(shapes);
                }
                if (ShapePropertyMappingPanel.PROP_COMBOBOX_MAPPING_VALUE.equals(boxName)) {
                    for (Shape shape : dataObject.getShapes()) {
                        shape.setValueFromAttribute((String) item);
                    }
                    Shape[] shapes = new Shape[scene.getSelectedObjects().size()];
                    scene.getSelectedObjects().toArray(shapes);
                    shapePropertiesPanel.setActiveShape(shapes);
                }
                if (ShapePropertiesPanel.PROP_COMBOBOX_IMPORT_TYPE.equals(boxName)) {
                    for (Object o : scene.getSelectedObjects()) {
                        ((Shape) o).setImportType((Shape.ImportType) item);
                    }
                }
                if (ShapePropertiesPanel.PROP_COMBOBOX_SHAPE_TYPE.equals(boxName)) {
                    for (Object o : scene.getSelectedObjects()) {
                        ((Shape) o).setType((Shape.ShapeType) item);
                    }
                }
            } else if (evt.getStateChange() == ItemEvent.DESELECTED) {
                // Item is no longer selected
            }
        }
    }
}
