/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.shapetype.preview;

import java.awt.Color;
import java.awt.event.ItemListener;
import java.util.Arrays;
import java.util.List;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.geometry.model.Shape;
import org.jdesktop.swingx.combobox.EnumComboBoxModel;
import org.jdesktop.swingx.combobox.ListComboBoxModel;
import org.openide.util.NbBundle;

/**
 * Panel which shows the properties of the active shapes Will show the
 * properties of the shapes set by setActiveShapes()
 *
 * @author Gerben Feenstra
 */
@NbBundle.Messages({
    "shape properties panel id label=Id",
    "shape properties panel name label=Name",
    "shape properties panel value label=Value",
    "shape properties panel number of shapes label=Selected shapes",
    "shape properties panel import_type label=Action",
    "shape properties panel shape_type label=Type"
})
public class ShapePropertiesPanel extends JPanel {

    public static final String PROP_COMBOBOX_IMPORT_TYPE = "importTypeBox";
    public static final String PROP_COMBOBOX_SHAPE_TYPE = "shapeTypeBox";
    // id
    private final JLabel idLabel = new JLabel(Bundle.shape_properties_panel_id_label());
    private final JLabel idText = new JLabel();
    // name
    private final JLabel nameLabel = new JLabel(Bundle.shape_properties_panel_name_label());
    private final JLabel nameText = new JLabel();
    // value
    private final JLabel valueLabel = new JLabel(Bundle.shape_properties_panel_value_label());
    private final JLabel valueText = new JLabel();
    // import type
    private final JLabel typeLabel = new JLabel(Bundle.shape_properties_panel_import_type_label());
    private final ListComboBoxModel<Shape.ImportType> typeModel = new EnumComboBoxModel<>(Shape.ImportType.class);
    private final JComboBox<Shape.ImportType> typeBox = new JComboBox<>(typeModel);
    // shape type
    private final JLabel shapeTypeLabel = new JLabel(Bundle.shape_properties_panel_shape_type_label());
    private final DefaultComboBoxModel shapeTypeModel = new DefaultComboBoxModel();
    private final JComboBox<Shape.ShapeType> shapeTypeBox = new JComboBox<>(shapeTypeModel);
    // number of shapes (multi selection)
    private final JLabel numberOfShapesLabel = new JLabel(Bundle.shape_properties_panel_number_of_shapes_label());
    private final JLabel numberOfShapesText = new JLabel();

    public ShapePropertiesPanel() {
        setLayout(new MigLayout("wrap 2"));
        typeBox.setName(PROP_COMBOBOX_IMPORT_TYPE);
        this.setBackground(Color.white);


        add(idLabel);
        add(idText);
        add(nameLabel);
        add(nameText);
        add(valueLabel);
        add(valueText);
        add(typeLabel);
        add(typeBox);
        if (shapeTypeModel.getSize() > 1) {
            add(shapeTypeLabel);
            add(shapeTypeBox);
        }
    }

    public void setActiveShape(Shape... shape) {
        if (shape.length > 1) {
            setMultiple(shape.length);
        } else if (shape.length == 1) {
            setSingle(shape[0]);
        }
    }

    public void setShapeTypes(List<Shape.ShapeType> shapeTypes) {
        shapeTypeModel.removeAllElements();
        for (Shape.ShapeType shapeType : shapeTypes) {
            shapeTypeModel.addElement(shapeType);
        }
    }

    /**
     * Set the multiple selection content for the properties panel When multiple
     * shapes are selected only that number of shapes are displayed in a label
     *
     * @param numberOfShapes the number of shapes that are selected.
     */
    private void setMultiple(int numberOfShapes) {
        if (!Arrays.asList(this.getComponents()).contains(numberOfShapesLabel)) {
            this.removeAll();
            add(numberOfShapesLabel);
            add(numberOfShapesText);
            this.repaint();
        }
        numberOfShapesText.setText(new Integer(numberOfShapes).toString());
    }

    /**
     * Set single selection content. When one shape is selected the id and name
     * are shown
     *
     * @param id the id to show
     * @param name the name to show
     */
    private void setSingle(Shape shape) {
        if (!Arrays.asList(this.getComponents()).contains(idLabel)) {
            this.removeAll();
            add(idLabel);
            add(idText);

            add(nameLabel);
            add(nameText);

            add(valueLabel);
            add(valueText);

            add(typeLabel);
            add(typeBox);

            if (shapeTypeModel.getSize() > 1) {
                add(shapeTypeLabel);
                add(shapeTypeBox);
            }
            this.repaint();
        }
        idText.setText(new Long(shape.getId()).toString());
        nameText.setText(shape.getName());
        String valueString = shape.getValue() != null ? shape.getValue().toString() : "Unknown";
        valueText.setText(valueString);
        typeModel.setSelectedItem(shape.getImportType());
        shapeTypeModel.setSelectedItem(shape.getType());
    }

    /**
     * Add a listener to the import type combo box
     *
     * @param listener
     */
    public void addImportTypeListener(ItemListener listener) {
        typeBox.addItemListener(listener);
        shapeTypeBox.addItemListener(listener);
    }

    /**
     * Remove an ItemListener from the import type combo box
     *
     * @param listener
     */
    public void removeImportTypeListener(ItemListener listener) {
        typeBox.removeItemListener(listener);
        shapeTypeBox.removeItemListener(listener);
    }
}
